(ns clojure-demo.core
  (:require
   ["@codemirror/language" :refer [LanguageSupport StreamLanguage]]
   ["@codemirror/legacy-modes/mode/yaml" :refer [yaml]]
   ["@codemirror/state" :as cm-state :refer [EditorState Transaction]]
   ["@codemirror/theme-one-dark" :refer [oneDark]]
   ["@codemirror/view" :as cm-view :refer [EditorView ViewUpdate]]
   ["@nextjournal/lang-clojure" :refer [clojure]]
   ["react-dom/client" :refer [createRoot]]
   ["tarts" :as tarts]
   [tick.core :as t]
   [cljs.core.async :as async]
   [cljs.core.async.interop :as async-in]
   [shadow.cljs.modern :refer [js-await]]
   [ajax.core :refer [GET raw-response-format]]
   [cl-eorg.html :as h :refer [body headers org->replacements]]
   [cl-eorg.parser :as o :refer [parse]]
   [cl-eorg.themes.tachyon :refer [tachyon-theme]]
   [clojure.string :as str]
   [clojure.data.xml :as xml]
   [honey.sql :as sql]
   [honey.sql.helpers :as sqlh]
   [reagent.core :as reagent]
   [reitit.coercion.spec :as rss]
   [reitit.frontend :as rf]
   [reitit.frontend.easy :as rfe]
   [reitit.frontend.history :refer [ignore-anchor-click?]]
   [sci.configs.reagent.reagent :as sci-reagent]
   [sci.configs.tonsky.datascript :as sci-datascript]
   [sci.configs.funcool.promesa :as sci-promesa]
   [sci.core :as sci]
   [clojure-demo.tailwind-theme :refer [tailwind-theme]]
   [spec-tools.data-spec :as ds]))

(def languages
  {"clojure" {:mode "clojure"}
   "clojurescript" {:mode "clojure"}
   "html" {:mode "html"}
   "json" {:mode "json"}
   "yaml" {:mode "yaml"}})

(def yaml-mode (LanguageSupport. (.define StreamLanguage yaml)))

(defn ^:sci/macro my-js-await [_ _ [name thenable] & body]
  (let [last-expr (last body)
        [body catch]
        (if (and (seq? last-expr) (= 'catch (first last-expr)))
          [(butlast body) last-expr]
          [body nil])]

    ;; FIXME: -> here will always return a promise so shouldn't be necessary to add js hint?
    `(-> ~thenable
         ~@(when (seq body)
             [`(.then (fn [~name] ~@body))])
         ~@(when catch
             (let [[name & body] catch]
               [`(.catch (fn [~name] ~@body))]
               )))))

;; https://github.com/babashka/sci.configs
(def rf-ns (sci/create-ns 'reitit.frontend nil))
(def rfe-ns (sci/create-ns 'reitit.frontend.easy nil))
(def rss-ns (sci/create-ns 'reitit.coercion.spec nil))
(def sql-ns (sci/create-ns 'honey.sql.core nil))
(def sqlh-ns (sci/create-ns 'honey.sql.helpers nil))
(def shadow-ns (sci/create-ns 'shadow.cljs.modern nil))
(def xml-ns (sci/create-ns 'clojure.data.xml nil))
(def tick-ns (sci/create-ns 'tick.core nil))

;; core async does not work with sci
(def async-ns (sci/create-ns 'cljs.core.async nil))
(def async-in-ns (sci/create-ns 'cljs.core.async.interop nil))

(def sci-ctx
  (sci/init
   {:classes {'js js/globalThis :allow :all}
    :features #{:cljs}
    :namespaces
    {'reagent sci-reagent/reagent-namespace
     'promesa sci-promesa/promesa-namespace
     ;'h {'html (sci/copy-var h/html hc-ns)}
     ;'async {'go (sci/copy-var async/go async-ns)}
     ;'shadow {'js-await (sci/copy-var js-await shadow-ns)}
     'shadow {'js-await (sci/copy-var my-js-await shadow-ns)}
     ;'async-in {'<p! (sci/copy-var async-in/<p! async-in-ns)}
     'd sci-datascript/core-namespace
     'tick {'format (sci/copy-var t/format tick-ns)
            'zoned-date-time (sci/copy-var t/zoned-date-time tick-ns)}
     'xml {'emit-str (sci/copy-var xml/emit-str xml-ns)
           ;; dont think indent-str & alias uri is supported in clojurescript
           ;;'indent-str (sci/copy-var xml/indent-str xml-ns)
           ;;'alias-uri (sci/copy-var xml/alias-uri xml-ns)
           }
     'sql {'format (sci/copy-var sql/format sql-ns)
           #_#_'raw (sci/copy-var sql/raw sql-ns)}
     'sqlh {'select (sci/copy-var sqlh/select sqlh-ns)
            'from (sci/copy-var sqlh/from sqlh-ns)
            'limit (sci/copy-var sqlh/limit sqlh-ns)
            'join (sci/copy-var sqlh/join sqlh-ns)
            'values (sci/copy-var sqlh/values sqlh-ns)
            'on-conflict (sci/copy-var sqlh/on-conflict sqlh-ns)
            'do-update-set (sci/copy-var sqlh/do-update-set sqlh-ns)
            'insert-into (sci/copy-var sqlh/insert-into sqlh-ns)
            'order-by (sci/copy-var sqlh/order-by sqlh-ns)
            'group-by (sci/copy-var sqlh/group-by sqlh-ns)
            'where (sci/copy-var sqlh/where sqlh-ns)}
     'rf {'router (sci/copy-var rf/router rf-ns)}
     'rss {'coercion (sci/copy-var rss/coercion rss-ns)}
     'rfe {'start! (sci/copy-var rfe/start! rfe-ns)
           'href (sci/copy-var rfe/href rfe-ns)}}}))

;(def sci-ctx  (sci/empty-environment))
(sci/alter-var-root sci/print-fn (constantly *print-fn*))

(defn slugify [s]
  (when s
    (str
     (-> s
         (str/lower-case)
         (str/replace #"[^\w]+" "-")
         (str/replace #"^-\\|-\\-$" "")))))

(defn fetch-selected-text
  "Get the users selected text"
  [updated-view transactions]
  (reduce (fn [text t]
            (if (= "select.pointer" (str (.annotation t (.-userEvent Transaction))))
              (conj text
                    (.sliceDoc
                     ^EditorState (.-state updated-view)
                     (.-from (.-main (.-selection t)))
                     (.-to (.-main (.-selection t)))))
              text))
          []
          transactions))

(defn err-boundary
  "https://github.com/reagent-project/reagent/blob/master/doc/ReactFeatures.md#error-boundaries"
  [children]
  (let [err-state (reagent/atom nil)]
    (reagent/create-class
     {:display-name "Error Boundary"
;      :component-did-catch (fn [this err info] #_(reset! err-state [err info]))
      :get-derived-state-from-error (fn [e]
                                      (reset! err-state e #_[err info])
                                      #js {})
      :reagent-render (fn [children]
                        (if @err-state
                          [:div (str @err-state)]
                          children
                          ;(into [:<>] children)
                          ))})))
(defn code-editor
  [{:keys [exports class results]} content]
  (let [language class
        editor (atom nil)
        evaled-result (reagent/atom nil)
        update-timeout (reagent/atom nil)
        update-fn (.of (.-updateListener EditorView)
                       (fn [^ViewUpdate view-update]
                         (when (some #(= language %) ["clojure" "clojurescript"])
                           (js/clearTimeout @update-timeout)
                           (reset! update-timeout
                                   (js/setTimeout
                                    (fn []
                                      (let [selected-text (fetch-selected-text view-update (.-transactions view-update))
                                            eval-code (if (seq selected-text)
                                                        (first selected-text)
                                                        (.-doc (.-state view-update)))]
                                        (reset! evaled-result (sci/eval-string* sci-ctx
                                                                                (.toString eval-code)))
                                        (prn "updated delayed"))) 1000)))))
        start-state
        (.create EditorState
                 (clj->js {:doc content
                           :mode "text/x-clojure"
                           ;:mode "text/yaml"
                           :extensions [(clojure) yaml-mode update-fn oneDark #_cm-keymap/of #_cm-commands/default-keymap]}))
        view (atom nil)]
    (reagent/create-class
     {:component-did-mount
      (fn [_]
        (reset! view (EditorView.
                      (clj->js  (merge {} #_(get languages language)
                                       {:state start-state
                                        ;:mode "yaml"
                                        :mode "clojure"
                                        :updateListener prn
                                        :parent @editor})))))
      :component-will-unmount (fn [_] (.destroy @view))
      :reagent-render (fn []
                        (if (= exports "none") 
                          nil
                          [:div.mb-8
                           [:div.ba.ma-0.f5.b--black-05.pa2.overflow-auto.editor {:ref #(reset! editor %)}]
                           (when @evaled-result
                             [err-boundary
                              [:pre.border-2.pa-4.pl-6.text-xl
                               {:style {:white-space "pre-line"}}
                               ;; See org mode :results key
                               (case results
                                 "verbatim" (str @evaled-result)
                                 "value" (str @evaled-result)
                                 "output" @evaled-result
                                 "silent" ""
                                 @evaled-result)]])]))})))

(def theme-tachyon
  (merge tachyon-theme
         {:SRC code-editor
          :HEADER1 (fn [v _]  [:h1 {:id (slugify v)} [:a {:name (slugify v)} v]])
          :HEADER2 (fn [v _] [:h2 {:id (slugify v)} [:a {:name (slugify v)} v]])
          :HEADER3 (fn [v _] [:h2 {:id (slugify v)} [:a {:name (slugify v)} v]])
          :LINE :p.f5.f5-ns.lh-copy.mt0
          :BULLETS :ul.mt0
          :LINK :a #_link-handler}))

(def theme
  (merge tailwind-theme
         {:SRC code-editor
          :HEADER1 (fn [v _]  [:h1.text-2xl.fw6.lh-title.mt-0.mb-2 {:id (slugify v)} [:a {:name (slugify v)} v]])
          :HEADER2 (fn [v _] [:h2.text-xl.fw6.lh-title.mt-0.mb-2 {:id (slugify v)} [:a {:name (slugify v)} v]])
          :HEADER3 (fn [v _] [:h2.text-base.lh-title.mt-0.mb-2 {:id (slugify v)} [:a {:name (slugify v)} v]])
          :LINE :p.f5.f5-ns.lh-copy.mt0
          :BULLETS :ul.mt0
          :LINK :a #_link-handler}))

#_(def theme-toc-tachyon
  (merge tachyon-theme
         {:HEADER1 (fn [v _] [:li [:a {:href (str "#" (slugify v))}
                                   [:span.f4.fw6.f3-ns.lh-title.mt0.mb2 v]]])
          :HEADER2 (fn [v _] [:li.ml2 [:a {:href (str "#" (slugify v))}
                                       [:span.f4.fw6.f4-ns.lh-title.mt0.mb2 v]]])
          :HEADER3 (fn [v _] [:li.ml4 [:a {:href (str "#" (slugify v))}
                                       [:span.f5.fw6.f5-ns.lh-title.mt0.mb2 v]]])
          :HEADER4 (fn [v _] [:li.ml6 [:a {:href (str "#" (slugify v))}
                                       [:span.f6.fw6.f5-ns.lh-title.mt0.mb2 v]]])
          :HEADER5 (fn [v _] [:li.ml8 [:a {:href (str "#" (slugify v))}
                                       [:span.f6.fw6.f5-ns.lh-title.mt0.mb2 v]]])
          :HEADER6 (fn [v _] [:li.ml10 [:a {:href (str "#" (slugify v))}
                                        [:span.f6.fw6.f5-ns.lh-title.mt0.mb2 v]]])}))

(def theme-toc
  (merge tailwind-theme
         {:HEADER1 (fn [v _] [:li.list-decimal [:a.text-blue-800 {:class "hover:text-blue-500" :href (str "#" (slugify v))}
                                   [:span.text-xl.fw6.lh-title.mt0.mb2 v]]])
          :HEADER2 (fn [v _] [:li.list-decimal.ml-4 [:a.text-blue-800 {:href (str "#" (slugify v))}
                                        [:span.text-lg.fw6.lh-title.mt0.mb2 v]]])
          :HEADER3 (fn [v _] [:li.list-decimal.ml-8 [:a.text-blue-800 {:class "hover:text-blue-500" :href (str "#" (slugify v))}
                                        [:span.text-base.fw6.lh-title.mt0.mb2 v]]])
          :HEADER4 (fn [v _] [:li.list-decimal.ml-12 [:a.text-blue-800 {:href (str "#" (slugify v))}
                                         [:span.text-sm.fw6.lh-title.mt0.mb2 v]]])
          :HEADER5 (fn [v _] [:li.list-decimal.ml-26 [:a.text-blue-800 {:href (str "#" (slugify v))}
                                         [:span.text-xs.fw6.f5-ns.lh-title.mt0.mb2 v]]])
          :HEADER6 (fn [v _] [:li.list-decimal.ml-30 [:a.hover:decoration-blue-400 {:href (str "#" (slugify v))}
                                         [:span.text-xs.fw6.f5-ns.lh-title.mt0.mb2 v]]])}))

;; put constant data here
(def site-data
  {:homepage {:intro "Clojure tutorials examples and exploration"}
   :dslpage {:intro "A domain-specific language (DSL) is a language designed to be used for a specific task or domain, clojure has a rich set of DSL some popular DSL's are listed on this page with example's on usage. "}
   :lorem "Lorem Ipsum is simply dummy text of the printing and typesetting industry. Lorem Ipsum has been the industry's standard dummy text ever since the 1500s, when an unknown printer took a galley of type and scrambled it to make a type specimen book. It has survived not only five centuries, but also the leap into electronic typesetting, remaining essentially unchanged. It was popularised in the 1960s with the release of Letraset sheets containing Lorem Ipsum passages, and more recently with desktop publishing software like Aldus PageMaker including versions of Lorem Ipsum."
   :pages {:dialects {:title "Dialects & Interop"
                      :intro "Clojure has the ability to run on multiple technology stacks, this allows code reuse and consistency regardless of which eco system you want to make use of."
                      :key ::dialects
                      :demos [{:title "Clojure"
                               :description "The original language running on the jvm with access to the java eco system"
                               :icon-image "https://clojure.org/images/clojure-logo-120b.png"
                               :page "clojure-info"}
                              {:title "Clojurescript"
                               :description "Clojure running in the browser or on top of node, with access to the js eco system."
                               :icon-image "https://clojurescript.org/images/cljs-logo-60b.png"
                               :page "clojurescript-info"}
                              {:title "ClojureCLR"
                               :description "Clojure running on microsofts CLR with access to .net ecosystem"
                               :icon-image "https://clojure.org/images/clojure-logo-120b.png"

                               :page "clojureclr-info"}
                              {:title "ClojureDart"
                               :description "Clojure for dart, use clojure to build apps using flutter"
                               :page "clojuredart-info"}]}
           :dsl {:title "DSL's"
                 :key ::dsl
                 :intro "A domain-specific language (DSL) is a language designed to be used for a specific task or domain, clojure has a rich set of DSL some popular DSL's are listed on this page with example's on usage. "
                 :demos [{:title "Hiccup HTML Demo"
                          :page "hiccup-dsl-demo"
                          :description "Hiccup is a DSL for generating HTML it uses data structures over strings to generate html, this makes it far easier to dynamically generate html by using the language constructs to manipulate the tree."
                         ; :link (rfe/href ::demo {:page "hiccup-dsl-demo"})
                          :icon-image "https://miro.medium.com/max/1400/1*CEYFj5R57UFyCXts2nsBqA.png"}
                         {:title "Honey SQL Demo"
                          :page "honey-sql-demo"
                          :description "Similar to honey but for SQL allows you to split apart your complex queries into parts and compose them together."
                       ;   :link (rfe/href ::demo {:page "honey-dsl-demo"})
                          :icon-image "https://miro.medium.com/max/1400/1*CEYFj5R57UFyCXts2nsBqA.png"}
                         {:title "Datalog Demo"
                          :page "datalog-demo"
                          :description "Datalog is a popular DSL with in the clojure for querying deductive database system's"
                       ;   :link (rfe/href ::demo {:page "datalog-demo"})
                          :icon-image "https://raw.githubusercontent.com/tonsky/datascript/master/extras/logo.svg"}]}
           :devops {:title "Deployment & testing"
                    :intro "Running, Testing and deploying your software in a CI pipeline"
                    :key ::devops
                    :demos [{:title "CI Demo"
                             :description "Clojure in a CI pipeline, building artifacts running tests & deploying"
                             :page "ci-demo"
                             ;;:link (rfe/href ::demo {:page "ci-demo"})
                             :icon-image "https://avatars.githubusercontent.com/u/2181346?s=200&v=4"}]}
           :examples {:title "Examples"
                      :intro "Some example applications"
                      :key ::examples
                      :demos [{:title "Maps"
                               :description "Some quick examples using frontend map api's like google maps."
                               :page "example-maps"
                               :icon-image "https://avatars.githubusercontent.com/u/2181346?s=200&v=4"}
                              {:title "XML sitemap"
                               :description "Using data.xml to generate a sitemap"
                               :page "example-xml-sitemap"
                               :icon-image "https://avatars.githubusercontent.com/u/2181346?s=200&v=4"}]}
           :terminology {:title "Terminology"
                         :intro ""
                         :key ::terminology}
           }
   :demos
   {:hiccup-dsl-demo
    {:file "documents/hiccup-dsl-demo.org" :git-link "https://github.com/atomjuice/dsl-demo"}
    :datalog-demo
    {:file "documents/datalog-demo.org" :git-link "https://github.com/atomjuice/dsl-demo"}
    :honey-sql-demo
    {:file "documents/honey-sql-demo.org" :git-link "https://github.com/atomjuice/dsl-demo"}
    :reagent-demo
    {:file "documents/reagent-reitit.org" :git-link "https://github.com/atomjuice/dsl-demo"}
    :clojure-basics
    {:file "documents/clojure-basics.org" :git-link "https://github.com/atomjuice/dsl-demo"}
    :ci-demo
    {:file "documents/ci-demo.org" :git-link "https://github.com/atomjuice/dsl-demo"}
    :example-maps
    {:file "documents/examples-maps.org" :git-link ""}
    :example-xml-sitemap
    {:file "documents/examples-xml-sitemap.org" :git-link "https://github.com/atomjuice/dsl-demo"}
    :containers
    {:file "documents/containers.org" :git-link "https://github.com/atomjuice/containers"}}})

;; Store site state
(defonce site-state (reagent/atom {}))

;; form one component to render an article
(defn article [{:keys [title description tagline]}]
  [:article.prose {:data-name "article-full-bleed-background"}
   [:div.cf {:style {:background "url(http://placekitten.com/g/600/300)"
                     :no-repeat "center center fixed" :background-size "cover"}}
    [:div.fl.pa3.pa4-ns.bg-white.black-70.measure-narrow.f3.times
     [:header.b--black-70.pv4 {:class (when tagline "bb")}
      [:h3.f2.fw7.ttu.tracked.lh-title.mt0.mb3.avenir title]
      (when tagline [:h4.f3.fw4.i.lh-title.mt0 tagline])]
     [:section.pt5.pb4 [:p.times.lh-copy.measure.f5.mt0 description]]]]])

;; form one component to render article tiles
(defn articles [{:keys [title body articles]}]
  [:section.mw7.center.avenir
   [:h2.text-4xl.mt-4.mb-4 title]
   (map (fn [{:keys [title author link description img-src img-alt] :as article}]
          [:article.bt.bb.b--black-10.border-b-2 {:key title}
           [:a {:href link}
            [:div.flex.flex-column.flex-row-ns.p-4
             [:div.flex-none.w-32
              (when img-src
                [:img.w-32 #_h-24.block {:src img-src :alt img-alt}])]
             [:div.ml-4.flex-grow #_{:class "w-2/3"}
              [:h1.text-2xl title]
              [:p description]
              [:p author]]]]])
        articles)])

;; form one component to render a nav link
(defn navbar-link [{:keys [href title text key] :or {text nil title nil}}]
  [:a.link.m-2 {:key (or key href) :class "hover:text-grey-200" :href href :title title} text])

;; form one component to render a navbar
(defn navbar [links]
  [:header.text-white.bg-zinc-900.p-6.pl-12.w-full
   (into [:nav.uppercase.font-medium.text-sm] (mapv navbar-link links))]
  #_[:header.bg-black-90.w-100.ph3.pv3.pv4-ns.ph4-m.ph5-l
     (into [:nav.f6.fw6.ttu.tracked] (mapv navbar-link links))])

(defn footer []
  [:footer.bg-zinc-800.text-slate-400.p-8.m-auto.w-full
   [:div.m-auto {:class "w-4/5"}
    [:a.hover:text-slate-200
     {:target "_blank" :href "https://matrix.to/#/@oly:matrix.org"}
     "Contact me"]]]
  #_[:div
     [:footer.bg-near-black.white-80.pv4.ph3.ph5-m.ph6-l.mid-gray.w-full
      [:a.f6.dib.ph2.link.mid-gray.dim
       {:target "_blank" :href "https://matrix.to/#/@oly:matrix.org"}
       "Contact me"]]])

(def toc (partial contains? (into #{} (map keyword [:HEADER1 :HEADER2 :HEADER3 :HEADER4 :HEADER5 :HEADER6]))))
(def org-code (partial contains? (into #{} (map keyword [:SRC]))))

(defn build-file-tar [code-blocks]
  (tarts (clj->js (mapv (fn [block] {:name (str (:tangle (second block)))
                                     :content (str (last block))})
                        code-blocks))))

(defn build-file-tar-hm [code-blocks]
  (tarts (clj->js code-blocks)))

(defn build-tarts-map [blocks]
  (->> blocks
       (group-by (fn [block] (:tangle (second block))))
       (reduce (fn [m [k v]]
                 (conj m {:name (str k)
                          :content (str (str/join "\n\n" (mapv last v)))})) [])))

(defn org->split2
  "Split out meta and body"
  [dsl]
  {:header (filter (fn filter-headers [tag] (headers (first tag))) dsl)
   :toc (filter (fn filter-toc [tag] (toc (first tag))) dsl)
   :code (filter (fn filter-code [tag] (org-code (first tag))) dsl)
   :body (filter (fn filter-body [tag] (body (first tag))) dsl)})


(defn home-page []
  [:<>
   [articles
    {:title "Clojure Demos"
     :body (-> site-data :homepage :intro)
     :articles
     [{:title "Clojure Basics"
       :description "Getting started with clojure syntax datatype's sequences conditions"
       :link (rfe/href ::page {:page "clojure-basics"})
       :img-src "https://clojure.org/images/clojure-logo-120b.png"}
      {:title "Reagent Demo"
       :description "React application using reagent"
       :link (rfe/href ::page {:page "reagent-demo"})
       :img-src "https://raw.githubusercontent.com/reagent-project/reagent/master/logo/logo-text.png"}]}]])

(defn grouped-page [route]
  (let [group (keyword (name (:name (:data route))))]
    [:<>
     [articles
      {:title (-> site-data :pages group :title)
       :body (-> site-data :pages group :intro)
       :articles
       (mapv (fn fmt-map [demo]
               {:title (:title demo)
                :description (:description demo)
                :link (rfe/href ::page {:page (:page demo)})
                :img-src (:icon-image demo)})
             (-> site-data :pages group :demos))}]]))

(defn default-page [route]
  (let [demo-key (keyword (-> route :parameters :path :page))
        content (reagent/atom {})]
    (GET (str "/" (-> site-data :demos demo-key :file))
      {:response-format (raw-response-format)
       :handler (fn [response]
                  ;;(prn (org->replacements theme (parse response) ))
                  ;;[:LINE "" [:LINK {:href "https://github.com/weavejester/hiccup"} "https://github.com/weavejester/hiccup"]]
                  (->> response
                       parse
                       org->split2
                       (reset! content)))})
    (fn [route]
      (if @content
        [:main
         [:h1.mt-8.mb-8.text-4xl (:content (last (first (:header @content))))]
         [:div.mw7.center.avenir
          (into [:ol.list-inside.m-6.font-semibold] (org->replacements theme-toc (:toc @content)))]
         [:p "The code in these examples is evaluated when modified, you can highlight a partial expression to evalute the selection, You can also download the code as a tar if you like and use it in your favourite editor."]
         [:a.bg-sky-500.text-white.m-2.p-2.pl-4.pr-4.inline-block
          {:download (str (slugify (:content (last (first (:header @content))))) ".tar")
           :title (:content (last (first (:header @content))))
           :href (.createObjectURL
                  js/URL
                  (js/Blob. #js [(build-file-tar-hm
                                  (build-tarts-map
                                   (:code @content)))]
                            {:type "application/tar"}))}
          "Download Code"]
         [:div
          (into [:div] (org->replacements theme (:body @content)))]]
        [:<>]))))



;; form one render about page component
(defn about-page []
  [:div
   [:section.mw7.center.avenir
    [:h1 "Clojure library examples to aid learning"]
    [:p ""]
    [article {:title "Example Article component"
              :description (-> site-data :lorem)
              :tagline "https://tachyons.io/components/"}]]])

;; define our page routes passed into reitit later on
(def routes
  [["/"
    {:name ::frontpage
     :my-data "hi"
     :view home-page}]

   ["/page/:page"
    {:name ::page
     :view default-page
      :parameters {:path {:page string?}
                    :query {(ds/opt :foo) keyword?}}}]

   ["/terminology/"
    {:name ::terminology
     :view grouped-page}]

   ["/dialects/"
    {:name ::dialects
     :view grouped-page}]

   ["/examples/"
    {:name ::examples
     :view grouped-page}]

   ["/devops/"
    {:name ::devops
     :view grouped-page}]

   ["/dsl/"
    {:name ::dsl
     :view grouped-page}]

   ["/about/"
    {:name ::about
     :view about-page}]

   #_["/demo/:page"
      {:name ::demo
       :view demo-page
       :parameters {:path {:page string?}
                    :query {(ds/opt :foo) keyword?}}}]])

;; top level component contains nav and adds in the select page into a containing element
;; we are adding in a style sheet but this will often be done in index.html
(defn current-page []
  [:div
   [navbar (concat
            [{:href (rfe/href ::frontpage) :title "title here" :text "home" :key "homepage"}]

            (mapv (fn build-nav [[key page]] {:href (rfe/href (:key page)) :text (:title page)}) (-> site-data :pages))
            [;;{:href (rfe/href ::dsl) :text "DSL's" :key "dsl"}
             {:title "Domain Specific Languages" :href (rfe/href ::about) :text "About" :key "about"}
             #_{:href (rfe/href ::i-do-not-exist) :text "missing"}])]
   [:main.m-auto {:class "w-3/5"}
    (when-let [view (-> @site-state :current-route :data :view)] [view (-> @site-state :current-route)])]
   [footer]])

;; This simply calls reagent render and puts the result in a div with the id of app
;; you can create your own index.html or figwheel provides one with the app id which will replace the default data
;; ^:after-load is meta data its not needed but informs figwheel to run this code after a page load

(defn mount-root-page []
  ;; this select the main node from the html file and injects your page content
  (.render
   (createRoot (.getElementById js/document "app"))
   (reagent/as-element [err-boundary [current-page]])))

(defn ^:after-load render-site []
  ;; this select the main node from the html file and injects your page content
  (mount-root-page))

(defn ^:dev/after-load startup! []
  (rfe/start!
   (rf/router routes {:data {:coercion rss/coercion}})
   (fn [m] (swap! site-state assoc :current-route m))
   ;; set to false to enable HistoryAPI
   {:use-fragment false
    :ignore-anchor-click?
    (fn [router e el uri]
      ;; Add additional check on top of the default checks\
      (and
       (ignore-anchor-click? router e el uri)
       (not (let [href (or (.-href el) "")
                  result (str/includes? href "#")]
              #_(when result
                ;(.preventDefault e)
                  #_(js/console.log "will prevent by href" href))
              result))))})
  (render-site))

;; we defonce the startup so that hot reloading does not reinitialize the state of the site
(def launch (do (startup!) true))

(comment
  @site-state
  (org->replacements tachyon-theme [[:SRC {:LANGUAGE "shell"} "hi"]])

  (GET "/test.org" {:handler (fn [response] (swap! site-state assoc :content response))}))
